#! /usr/bin/env perl
#
# convert a marked-up autoconf m4 macro into formatted HTML
#
# This script follows the public domain macro2html.cpp part 
# from Peter Simons <simons@computer.org> being used for the same purpose 
# in ac-archive for a while. Since genindex.pl is already used and a perl 
# script, we can already rely on perl as a prerequisite - and it is better 
# given otherwise c++ compiler is a prerequisite to build the web pages.

use strict;

sub escape_html
{
    my $text = $_[0];
    $text =~ s,\&,\&amp\;,g;
    $text =~ s,\",\&quot\;,g;
    $text =~ s,\<,\&lt\;,g;
    $text =~ s,\>,\&gt\;,g;
    return $text;
}

sub escape_blanks
{
    my $text = $_[0];
    $text =~ s,\s,\&nbsp\;,g;
    return $text;
}

############################################################## main #######

# We expect one parameter on the command line: The name of the
# macro contained in the file we process.
if ($#ARGV < 0)
{
    print STDERR "** ERROR: Wrong command line syntax.\n";
    print STDERR "Usage: macro2html macro-name\n";
    exit 1;
}

# These are the parts that a macro should contain.
my ($synopsis, $version, $author);
my ($documentation, $macro);

# Parse the macro's header for the documentation and the
# keywords;

my $pat_comment = '^dnl[ \\t]*(.*)[ \\t]*$';
my $pat_verbatim_comment = '^dnl[ \\t][ \\t](.*)[ \\t]*$';
my $pat_synopsis = '^dnl[ \\t]+@synopsis[ \\t]+(.*)[ \\t]*$';
my $pat_version = '^dnl[ \\t]+@version[ \\t]+(.*)[ \\t]*$';
my $pat_author = '^dnl[ \\t]+@author[ \\t]+(.*)[ \\t]*$';

my $last = "";
my $tail = 0;
for (<STDIN>)
{
    chomp;

    if ($tail)
    {
	$macro .= escape_html($_)."\n";
	next;
    }

    if (m,^dnl\s+\@synopsis\s+(.*)\s*$,i) # $pat_synopsis
    {
	$synopsis .= "<br>\n" if length $synopsis;
	$synopsis .= escape_html($1);
	$last = "synopsis";
    }
    elsif (m,^dnl\s+\@author\s+(.*)\s*$,i) # $pat_author
    {
	$author = escape_html($1);
	$last = "author";
    }
    elsif (m,^dnl\s+\@version\s+(.*)\s*$,i) # $pat_version
    {
	my $line = $1;
	if ($line =~ m,^\$[I][d]: [^ ]+ ([^ ]+) ([^ ]+),i)
	{
	    $version = escape_html($1)." (".escape_html($2).")";
	}else{
	    $version = escape_html($line);
	}
	$last = "version";
    }
    elsif (m,^dnl\s+(obsoleted[:\s].*)\s*$,i) # obsoleted
    {
	$documentation .= "<br>" if $last eq "text";
	$documentation .= "<b>".escape_html($1)."</b>";
	$documentation .= "<br>\n";
	$last = "enum";
    }
    elsif (m,^dnl\s+([*-]\s.*)\s*$,i) # enumeration
    {
	$documentation .= "<br>" if $last eq "text";
	$documentation .= escape_html($1);
	$documentation .= "<br>\n";
	$last = "enum";
    }
    elsif (m,^dnl\s\s(.*)\s*$,i) # $pat_verbatim_comment
    {
	$documentation .= "<br>" if $last eq "text";
	$documentation .= "<code>&nbsp;&nbsp;";
	$documentation .= escape_blanks(escape_html($1));
	$documentation .= "</code><br>\n";
	$last = "code";
    }
    elsif (m,^dnl\s*(.*)\s*$,i) # $pat_comment
    {
	if (not length $1) {
	    $documentation .= "<p>";
	    $last = "para";
	}else{
	    $documentation .= escape_html($1)."\n";
	    $last = "text";
	}
    }else{
	$macro = escape_html($_)."\n";
	print STDERR ".";
	$tail = 1;
	# The header part is over. Now the remainder of the file is the
	# macro source code.
	$last = "over";
    }
}

my $archive = "";
if (length $ARGV[1])
{
    $archive = " &nbsp; &nbsp; <small>$ARGV[1]</small>";
}

# Print the formatted HTML page.

print "<html>","\n";
print "<head>","\n";
print "    <title>Macro: ",$ARGV[0],"</title>","\n";
print "</head>","\n";

print "<body>","\n";
my $_width100=' width="100%"';
my $_width50=' width="50%"';
my $_center=' align="center"';
my $O = '&nbsp;';
if (length $ARGV[0] > 32)
{
    print "<table$_width100>","\n";
    print "    <tr>","\n";
    print "    <td$_width50$_center><a href=\"../\">",
    "Back to the Main Page.</a></td>","\n";
    print "    <td$_width50$_center><a href=\"",$ARGV[0],".m4\">",
    "Download the M4 Source.</a></td>","\n";
    print "    </tr>","\n";
    print "</table>","\n";
    print "<hr><p>","\n";
    
    print "<h1 align=\"center\">",$ARGV[0],"</h1>","\n";
}else{
    print "<table$_width100>","\n";
    print "    <tr>","\n";
    print "    <td$_center><a href=\"../\">",
    "Back${O}to${O}the Main${O}Page.</a></td>","\n";

    print "    <td$_center><h1>", $ARGV[0],"</h1></td>","\n";

    print "    <td$_center><a href=\"",$ARGV[0],".m4\">",
    "Download the${O}M4${O}Source.</a></td>","\n";
    print "    </tr>","\n";
    print "</table>","\n";
    print "<hr><p>","\n";
}

if (0) # original
{
    print "<h2>Synopsis</h2>","\n";
    print "<p style=\"margin-left:2cm\"><code >",$synopsis,"</code>","\n";
    
    print "<h2>Version</h2>","\n";
#   print "<p style=\"margin-left:2cm\">",$version,$archive,"","\n";
    print "<p style=\"margin-left:2cm\"><code>",$version,"</code>","\n";

    print "<h2>Author</h2>","\n";
    print "<p style=\"margin-left:2cm\">",$author,"\n";

    print "<h2>Description</h2>","\n";
    print "<div style=\"margin-left:2cm\">","\n";
    print $documentation,"\n";
    print "</div>","\n";

    print "<h2>M4 Source Code</h2>","\n";
    print "<div style=\"margin-left:2cm\">","\n";
    print "<pre>\n",$macro,"</pre>","\n";
    print "</div>","\n";
}else{ # our style
    print "<dl>\n";
    print "<dt><big><b>Synopsis</b></big>","\n</dt><dd>";
    print "<p><code >",$synopsis,"</code>","\n</dd>";
    
    print "<dt><big><b>Version</b></big>","\n</dt><dd>";
    print "<p>",$version,$archive,"","\n</dd>";
#   print "<p><code>",$version,"</code>","\n</dd>";

    print "<dt><big><b>Author</b></big>","\n</dt><dd>";
    print "<p>",$author,"\n</dd>";

    print "<dt><big><b>Description</b></big>","\n</dt><dd>";
    print "<div>","\n";
    print $documentation,"\n";
    print "</div>","\n</dd>";

    print "<dt><big><b>M4 Source Code</b></big>","\n</dt><dd>";
    print "<div>","\n";
    print "<pre>\n",$macro,"</pre>","\n";
    print "</div>","\n</dd>";
    print "</dl>";
}    

print "</body>","\n";
print "</html>","\n";
